# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

packages = \
['extrucal']

package_data = \
{'': ['*']}

install_requires = \
['altair-saver>=0.5.0,<0.6.0',
 'ipykernel>=6.9.1,<7.0.0',
 'numpy>=1.22.2,<2.0.0',
 'pandas>=1.4.1,<2.0.0',
 'sphinx-autoapi>=1.8.4,<2.0.0',
 'sphinx-rtd-theme>=1.0.0,<2.0.0']

setup_kwargs = {
    'name': 'extrucal',
    'version': '0.1.1',
    'description': 'Provides functions for calculating various parameters in extrusion processes',
    'long_description': '# extrucal\n\n**Author:** John Lee\n\n`extrucal` provides functions for calculating throughput in extrusion processes and for generating tables and plots for throughput with respect to screw RPM and channel depth in metering section.\n\nA large portion of arguments for the functions were given the typical values. The arguments for functions are as follows:\n screw size, channel depth, polymer melt density, screw RPM, screw pitch, flight width, number of flights\n\n## Installation\n\n`extrucal` can be installed PyPI using the following terminal command:\n\n```bash\n$ pip install extrucal\n```\n\n## Package Functions\n\n- `throughput()`\n  - This function calculates the extrusion throughput (Drag Flow) given the screw size, RPM, the channel depth of metering channel, and screw pitch\n  \n- `throughput_table()`\n  - This function generates a table containing the extrusion throughput with respect to channel depth and screw RPM\n  \n- `throughput_plot()`\n  - This function generates a plot containing the extrusion throughput with respect to channel depth and screw RPM\n\n## Usage\n\n`extrucal` can be used to calculate throughput in extrusion processes and to generate table and plot for throughput as a function of screw RPM and channel depth in the metering section of screw\n\n```python\nfrom extrucal.extrucal import throughput, throughput_table, throughput_plot\n```\n\n## Dependencies\n\n-   Python 3.9 and Python packages:\n\n    -   pandas==1.4.1\n    -   numpy==1.22.2\n    -   ipykernel==6.9.1\n    -   altair-saver==0.5.0\n\n## Documentation\n\nDocumentation `extrucal` can be found at [Read the Docs](https://extrucal.readthedocs.io/)\n\n## Contributing\n\nInterested in contributing? Check out the contributing guidelines. Please note that this project is released with a Code of Conduct. By contributing to this project, you agree to abide by its terms.\n\n## License\n\n`extrucal` was created by John Lee. It is licensed under the terms of the MIT license.\n\n## Credits\n\n`extrucal` was created with [`cookiecutter`](https://cookiecutter.readthedocs.io/en/latest/) and the `py-pkgs-cookiecutter` [template](https://github.com/py-pkgs/py-pkgs-cookiecutter).\n',
    'author': 'John Lee',
    'author_email': None,
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/johnwslee/extrucal',
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.9,<4.0',
}


setup(**setup_kwargs)
